import tkinter as tk
from tkinter import ttk
import os
from pygame import mixer
import mutagen
from mutagen.easyid3 import EasyID3
from mutagen.mp3 import MP3

class MP3Player:
    def __init__(self, root):
        self.root = root
        self.root.title("MP3 Player")
        self.root.geometry("500x700")
        mixer.init()
        
        self.bg_color = "#2B3A55"
        self.fg_color = "#E8F0F2"
        self.btn_color = "#3E5C76"
        
        self.root.configure(bg=self.bg_color)
        self.playing = False
        self.current_file = None
        
        self.current_dir = os.path.dirname(os.path.abspath(__file__))
        self.m3u_files = self.load_m3u_files()
        self.all_mp3s = self.load_all_mp3s()
        self.playlist = self.all_mp3s  # Standardmäßig alle MP3s
        self.current_index = 0
        
        self.tech_window = None
        
        self.create_gui()
        self.root.protocol("WM_DELETE_WINDOW", self.on_closing)
        
    def load_m3u_files(self):
        m3u_dict = {}
        for file in os.listdir(self.current_dir):
            if file.lower().endswith('.m3u'):
                m3u_path = os.path.join(self.current_dir, file)
                playlist = []
                with open(m3u_path, 'r', encoding='utf-8') as f:
                    for line in f:
                        line = line.strip()
                        if line and not line.startswith('#'):
                            mp3_path = line if os.path.isabs(line) else os.path.join(self.current_dir, line)
                            if os.path.exists(mp3_path) and mp3_path.lower().endswith('.mp3'):
                                playlist.append(mp3_path)
                if playlist:
                    m3u_dict[file] = playlist
        return m3u_dict
    
    def load_all_mp3s(self):
        mp3_list = []
        for file in os.listdir(self.current_dir):
            if file.lower().endswith('.mp3'):
                mp3_path = os.path.join(self.current_dir, file)
                mp3_list.append(mp3_path)
        if not mp3_list:
            return ["Keine .mp3 Dateien gefunden"]
        return mp3_list
    
    def update_playlist(self, event=None):
        selected = self.playlist_selector.get()
        self.listbox.delete(0, 'end')
        if selected == "Alle MP3s":
            self.playlist = self.all_mp3s
        else:
            self.playlist = self.m3u_files.get(selected, self.all_mp3s)
        
        for song in self.playlist:
            self.listbox.insert('end', os.path.basename(song))
        self.current_index = 0
        self.current_file = None
        self.stop()
    
    def create_gui(self):
        style = ttk.Style()
        style.theme_use('default')
        style.configure("Horizontal.TScale", background=self.bg_color, troughcolor=self.btn_color)
        style.configure("Horizontal.TProgressbar", background=self.btn_color, troughcolor=self.bg_color)
        
        self.tag_frame = tk.Frame(self.root, bg=self.bg_color)
        self.tag_frame.pack(pady=20, fill='x', padx=10)
        
        self.title_label = tk.Label(self.tag_frame, text="Titel: -", bg=self.bg_color, 
                                  fg=self.fg_color, font=("Arial", 12))
        self.title_label.pack()
        
        self.artist_label = tk.Label(self.tag_frame, text="Artist: -", bg=self.bg_color, 
                                   fg=self.fg_color, font=("Arial", 10))
        self.artist_label.pack()
        
        self.album_label = tk.Label(self.tag_frame, text="Album: -", bg=self.bg_color, 
                                  fg=self.fg_color, font=("Arial", 10))
        self.album_label.pack()
        
        progress_frame = tk.Frame(self.root, bg=self.bg_color)
        progress_frame.pack(pady=10, fill='x', padx=20)
        
        self.progress = ttk.Progressbar(progress_frame, style="Horizontal.TProgressbar",
                                      orient='horizontal', length=300, mode='determinate')
        self.progress.pack(fill='x', expand=True)
        
        self.time_label = tk.Label(progress_frame, text="0:00 / 0:00", bg=self.bg_color,
                                 fg=self.fg_color, font=("Arial", 10))
        self.time_label.pack()
        
        control_frame = tk.Frame(self.root, bg=self.bg_color)
        control_frame.pack(pady=20)
        
        self.prev_btn = tk.Button(control_frame, text="◄◄ Prev", command=self.prev_track,
                                bg=self.btn_color, fg=self.fg_color, relief='flat')
        self.prev_btn.pack(side='left', padx=5)
        
        self.play_btn = tk.Button(control_frame, text="▶ Play", command=self.play_pause,
                                bg=self.btn_color, fg=self.fg_color, relief='flat')
        self.play_btn.pack(side='left', padx=5)
        
        self.stop_btn = tk.Button(control_frame, text="■ Stop", command=self.stop,
                                bg=self.btn_color, fg=self.fg_color, relief='flat')
        self.stop_btn.pack(side='left', padx=5)
        
        self.next_btn = tk.Button(control_frame, text="▶▶ Next", command=self.next_track,
                                bg=self.btn_color, fg=self.fg_color, relief='flat')
        self.next_btn.pack(side='left', padx=5)
        
        self.tech_btn = tk.Button(control_frame, text="Tech. Info", command=self.show_tech_info,
                                bg=self.btn_color, fg=self.fg_color, relief='flat')
        self.tech_btn.pack(side='left', padx=5)
        
        volume_frame = tk.Frame(self.root, bg=self.bg_color)
        volume_frame.pack(pady=20, fill='x', padx=20)
        
        tk.Label(volume_frame, text="Lautstärke:", bg=self.bg_color, 
                fg=self.fg_color).pack(side='left', padx=5)
        
        self.volume_slider = ttk.Scale(volume_frame, from_=0, to=100, orient='horizontal',
                                     command=self.set_volume, style="Horizontal.TScale")
        self.volume_slider.set(50)
        self.volume_slider.pack(side='left', fill='x', expand=True)
        
        # Playlist-Auswahl (jetzt zwischen Lautstärke und Listbox)
        selector_frame = tk.Frame(self.root, bg=self.bg_color)
        selector_frame.pack(pady=10, fill='x', padx=10)
        
        tk.Label(selector_frame, text="Playlist:", bg=self.bg_color, 
                fg=self.fg_color).pack(side='left', padx=5)
        
        self.playlist_selector = ttk.Combobox(selector_frame, values=["Alle MP3s"] + list(self.m3u_files.keys()))
        self.playlist_selector.pack(side='left', fill='x', expand=True)
        self.playlist_selector.set("Alle MP3s")
        self.playlist_selector.bind('<<ComboboxSelected>>', self.update_playlist)
        
        self.listbox = tk.Listbox(self.root, bg=self.btn_color, fg=self.fg_color,
                                selectbackground="#1D2A44", font=("Arial", 10))
        self.listbox.pack(pady=20, fill='both', expand=True, padx=10)
        
        for song in self.playlist:
            self.listbox.insert('end', os.path.basename(song))
        
        self.listbox.bind('<<ListboxSelect>>', self.on_select)
    
    def play_pause(self):
        if not self.playlist or "Keine .mp3 Dateien gefunden" in self.playlist:
            return
        
        if not self.playing:
            if not self.current_file:
                self.current_index = 0
                self.current_file = self.playlist[self.current_index]
                mixer.music.load(self.current_file)
                mixer.music.play()
            else:
                mixer.music.unpause()
            self.playing = True
            self.play_btn.config(text="❚❚ Pause")
            self.update_tags()
            self.update_progress()
        else:
            mixer.music.pause()
            self.playing = False
            self.play_btn.config(text="▶ Play")
    
    def stop(self):
        mixer.music.stop()
        self.playing = False
        self.play_btn.config(text="▶ Play")
        self.progress['value'] = 0
        self.time_label.config(text="0:00 / 0:00")
    
    def next_track(self):
        if not self.playlist or "Keine .mp3 Dateien gefunden" in self.playlist:
            return
        self.stop()
        self.current_index = (self.current_index + 1) % len(self.playlist)
        self.current_file = self.playlist[self.current_index]
        mixer.music.load(self.current_file)
        mixer.music.play()
        self.playing = True
        self.play_btn.config(text="❚❚ Pause")
        self.listbox.select_clear(0, 'end')
        self.listbox.select_set(self.current_index)
        self.update_tags()
        self.update_progress()
    
    def prev_track(self):
        if not self.playlist or "Keine .mp3 Dateien gefunden" in self.playlist:
            return
        self.stop()
        self.current_index = (self.current_index - 1) % len(self.playlist)
        self.current_file = self.playlist[self.current_index]
        mixer.music.load(self.current_file)
        mixer.music.play()
        self.playing = True
        self.play_btn.config(text="❚❚ Pause")
        self.listbox.select_clear(0, 'end')
        self.listbox.select_set(self.current_index)
        self.update_tags()
        self.update_progress()
    
    def set_volume(self, val):
        volume = float(val) / 100
        mixer.music.set_volume(volume)
    
    def update_tags(self):
        try:
            audio = EasyID3(self.current_file)
            self.title_label.config(text=f"Titel: {audio.get('title', ['-'])[0]}")
            self.artist_label.config(text=f"Artist: {audio.get('artist', ['-'])[0]}")
            self.album_label.config(text=f"Album: {audio.get('album', ['-'])[0]}")
        except:
            self.title_label.config(text="Titel: -")
            self.artist_label.config(text="Artist: -")
            self.album_label.config(text="Album: -")
    
    def update_progress(self):
        if self.playing and self.current_file:
            audio = MP3(self.current_file)
            length = audio.info.length
            self.progress['maximum'] = length
            pos = mixer.music.get_pos() / 1000
            self.progress['value'] = pos
            
            current_min, current_sec = divmod(int(pos), 60)
            total_min, total_sec = divmod(int(length), 60)
            self.time_label.config(text=f"{current_min}:{current_sec:02d} / {total_min}:{total_sec:02d}")
            
            if pos >= length - 0.5:
                self.next_track()
            elif self.playing:
                self.root.after(100, self.update_progress)
    
    def on_select(self, event):
        if not self.listbox.curselection() or "Keine .mp3 Dateien gefunden" in self.playlist:
            return
        self.current_index = self.listbox.curselection()[0]
        self.current_file = self.playlist[self.current_index]
        self.stop()
        mixer.music.load(self.current_file)
        mixer.music.play()
        self.playing = True
        self.play_btn.config(text="❚❚ Pause")
        self.update_tags()
        self.update_progress()
    
    def show_tech_info(self):
        if not self.current_file or "Keine .mp3 Dateien gefunden" in self.current_file:
            return
            
        if self.tech_window is not None:
            self.tech_window.destroy()
            
        self.tech_window = tk.Toplevel(self.root)
        self.tech_window.title("Technische Informationen")
        self.tech_window.geometry("400x300")
        self.tech_window.configure(bg=self.bg_color)
        
        audio = MP3(self.current_file)
        
        info_text = tk.Text(self.tech_window, bg=self.btn_color, fg=self.fg_color,
                          font=("Arial", 10), height=15, wrap='word')
        info_text.pack(pady=10, padx=10, fill='both', expand=True)
        
        info = f"Dateiname: {os.path.basename(self.current_file)}\n"
        info += f"Dateigröße: {os.path.getsize(self.current_file) / 1024:.2f} KB\n"
        info += f"Dauer: {audio.info.length:.2f} Sekunden\n"
        info += f"Sample-Rate: {audio.info.sample_rate} Hz\n"
        info += f"Bitrate: {audio.info.bitrate / 1000} kbps\n"
        info += f"Kanäle: {audio.info.channels}\n"
        info += f"Encoder: {audio.info.encoder_info}\n"
        
        try:
            tags = EasyID3(self.current_file)
            if 'date' in tags:
                info += f"Erscheinungsjahr: {tags['date'][0]}\n"
            if 'genre' in tags:
                info += f"Genre: {tags['genre'][0]}\n"
        except:
            pass
            
        info_text.insert('end', info)
        info_text.config(state='disabled')
    
    def on_closing(self):
        mixer.music.stop()
        if self.tech_window is not None:
            self.tech_window.destroy()
        self.root.destroy()

if __name__ == "__main__":
    root = tk.Tk()
    app = MP3Player(root)
    root.mainloop()
